package de.intevation.flys.exports;

import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;

import org.apache.log4j.Logger;

import au.com.bytecode.opencsv.CSVWriter;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.CallContext;

import de.intevation.flys.artifacts.WINFOArtifact;
import de.intevation.flys.artifacts.model.WQKms;


/**
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public class WaterlevelExporter extends AbstractExporter {

    /** The logger used in this exporter.*/
    private static Logger logger = Logger.getLogger(WaterlevelExporter.class);


    public static final String CSV_KM_HEADER =
        "export.waterlevel.csv.header.km";

    public static final String CSV_W_HEADER =
        "export.waterlevel.csv.header.w";

    public static final String CSV_Q_HEADER =
        "export.waterlevel.csv.header.q";

    public static final String DEFAULT_CSV_KM_HEADER = "Fluss-Km";
    public static final String DEFAULT_CSV_W_HEADER  = "W [NN + m]";
    public static final String DEFAULT_CSV_Q_HEADER  = "Q [m³/s]";


    /** The storage that contains all WQKms objects for the different facets.*/
    protected List<WQKms[]> data;


    public void init(Document request, OutputStream out, CallContext context) {
        logger.debug("WaterlevelExporter.init");

        super.init(request, out, context);

        this.data = new ArrayList<WQKms[]>();
    }


    protected void addData(Artifact artifact) {
        data.add(getWaterlevelData(artifact));
    }


    /**
     * Returns the waterlevel data computed by the WINFOArtifact.
     *
     * @param artifact The WINFOArtifact.
     *
     * @return the computed waterlevel data.
     */
    protected WQKms[] getWaterlevelData(Artifact artifact) {
        WINFOArtifact winfoArtifact = (WINFOArtifact) artifact;
        WQKms[]       wqkms         = winfoArtifact.getWaterlevelData();

        logger.debug("Got " + wqkms.length + " WQKms objects.");

        return wqkms;
    }


    protected void writeCSVData(CSVWriter writer) {
        logger.info("WaterlevelExporter.writeData");

        writeCSVHeader(writer);

        for (WQKms[] tmp: data) {
            for (WQKms wqkms: tmp) {
                wQKms2CSV(writer, wqkms);
            }
        }
    }


    protected void writeCSVHeader(CSVWriter writer) {
        logger.info("WaterlevelExporter.writeCSVHeader");

        writer.writeNext(new String[] {
            msg(CSV_KM_HEADER, DEFAULT_CSV_KM_HEADER),
            msg(CSV_W_HEADER, DEFAULT_CSV_W_HEADER),
            msg(CSV_Q_HEADER, DEFAULT_CSV_Q_HEADER)
        });
    }


    protected void wQKms2CSV(CSVWriter writer, WQKms wqkms) {
        logger.debug("WaterlevelExporter.wQKms2CSV");

        int      size   = wqkms.size();
        double[] result = new double[3];

        for (int i = 0; i < size; i ++) {
            result = wqkms.get(i, result);

            writer.writeNext(new String[] {
                Double.toString(result[2]),
                Double.toString(result[0]),
                Double.toString(result[1])
            });
        }
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
